const bodyParser = require('body-parser')

const CollectionsInvitations = app => {
  app.use(bodyParser.json())
  const basePath = '/api/collections/:collectionId/invitations'
  const routePath = './routes/collectionsInvitations'
  const authBearer = app.locals.passport.authenticate('bearer', {
    session: false,
  })
  /**
   * @api {post} /api/collections/:collectionId/invitations Invite a user to a collection
   * @apiGroup CollectionsInvitations
   * @apiParam {collectionId} collectionId Collection id
   * @apiParamExample {json} Body
   *    {
   *      "email": "email@example.com",
   *      "role": "handlingEditor", [acceptedValues: handlingEditor, reviewer]
   *    }
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    {
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email@example.com",
   *      "teams": [
   *        "c576695a-7cda-4e27-8e9c-31f3a0e9d592"
   *      ],
   *      "username": "email@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": false,
   *      "editorInChief": false,
   *      "handlingEditor": true,
   *      "passwordResetToken": "04590a2b7f6c1f37cb84881d529e516fa6fc309c205a07f1341b2bfaa6f2b46c"
   *    }
   * @apiErrorExample {json} Invite user errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   *    HTTP/1.1 500 Internal Server Error
   */
  app.post(
    basePath,
    authBearer,
    require(`${routePath}/post`)(app.locals.models),
  )
  /**
   * @api {get} /api/collections/:collectionId/invitations/[:invitationId]?role=:role List collections invitations
   * @apiGroup CollectionsInvitations
   * @apiParam {id} collectionId Collection id
   * @apiParam {id} [invitationId] Invitation id
   * @apiParam {String} role The role to search for: handlingEditor, reviewer, author
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    [{
   *      "name": "John Smith",
   *      "timestamp": "123223121",
   *      "email": "email@example.com",
   *      "status": "pending",
   *      "invitationId": "1990881"
   *    }]
   * @apiErrorExample {json} List errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   */
  app.get(
    `${basePath}/:invitationId?`,
    authBearer,
    require(`${routePath}/get`)(app.locals.models),
  )
  /**
   * @api {delete} /api/collections/:collectionId/invitations/:invitationId Delete invitation
   * @apiGroup CollectionsInvitations
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {invitationId} invitationId Invitation id
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 204 No Content
   * @apiErrorExample {json} Delete errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 404 Not Found
   *    HTTP/1.1 500 Internal Server Error
   */
  app.delete(
    `${basePath}/:invitationId`,
    authBearer,
    require(`${routePath}/delete`)(app.locals.models),
  )
  /**
   * @api {patch} /api/collections/:collectionId/invitations/:invitationId Update an invitation
   * @apiGroup CollectionsInvitations
   * @apiParam {collectionId} collectionId Collection id
   * @apiParam {invitationId} invitationId Invitation id
   * @apiParamExample {json} Body
   *    {
   *      "isAccepted": false,
   *      "reason": "I am not ready" [optional]
   *    }
   * @apiSuccessExample {json} Success
   *    HTTP/1.1 200 OK
   *    {
   *      "id": "a6184463-b17a-42f8-b02b-ae1d755cdc6b",
   *      "type": "user",
   *      "admin": false,
   *      "email": "email@example.com",
   *      "teams": [],
   *      "username": "email@example.com",
   *      "fragments": [],
   *      "collections": [],
   *      "isConfirmed": false,
   *      "editorInChief": false,
   *      "handlingEditor": true,
   *      "passwordResetToken": "04590a2b7f6c1f37cb84881d529e516fa6fc309c205a07f1341b2bfaa6f2b46c"
   *    }
   * @apiErrorExample {json} Update invitations errors
   *    HTTP/1.1 403 Forbidden
   *    HTTP/1.1 400 Bad Request
   *    HTTP/1.1 404 Not Found
   *    HTTP/1.1 500 Internal Server Error
   */
  app.patch(
    `${basePath}/:invitationId`,
    authBearer,
    require(`${routePath}/patch`)(app.locals.models),
  )
}

module.exports = CollectionsInvitations
